import { Entity } from '../entity'
import { Storage } from '../storage'

/** @public */
export class EntityManager {
    private readonly _storage: Storage
    private _saveMap: Map<string, Entity<any>>
    private _removeMap: Map<string, Entity<any>>

    constructor(storage: Storage) {
        this._storage = storage
        this._saveMap = new Map()
        this._removeMap = new Map()
    }

    persist(entity: Entity<any>): this {
        const key = [entity.constructor.name, entity.getUniqKey()].join('&')
        this._saveMap.set(key, entity)
        return this
    }

    persistMany(entities: Entity<any>[]): this {
        for (let entity of entities) {
            this.persist(entity)
        }
        return this
    }

    remove(entity: Entity<any>): this {
        const key = [entity.constructor.name, entity.getUniqKey()].join('&')
        this._removeMap.set(key, entity)
        return this
    }

    forget(): this {
        this._saveMap = new Map()
        return this
    }

    async flush() {
        if (!this._saveMap.size && !this._removeMap.size) return this

        const session = this._storage.createSession()
        await session.start()
        await session.commit(async () => {
            for (let en of this._saveMap.values()) {
                await en._getRepo(this._storage).save(en)
            }
            this._saveMap = new Map()

            for (let en of this._removeMap.values()) {
                await en._getRepo(this._storage).remove(en)
            }
            this._removeMap = new Map()
        })

        return this
    }

    async refresh(entity: Entity<any>) {
        const repo = entity._getRepo(this._storage)
        return repo.findById(entity.getUniqKey())
    }
}
